@@ -9,7 +9,7 @@ import io
 import os
 import sys
 import warnings
-from unittest import TestCase, skipIf
+from unittest import TestCase
 
 try:
     from importlib import invalidate_caches   # Required from Python 3.3
@@ -23,9 +23,8 @@ from IPython import get_ipython
 from IPython.core import magic
 from IPython.core.error import UsageError
 from IPython.core.magic import (Magics, magics_class, line_magic,
-                                cell_magic, line_cell_magic,
-                                register_line_magic, register_cell_magic,
-                                register_line_cell_magic)
+                                cell_magic,
+                                register_line_magic, register_cell_magic)
 from IPython.core.magics import execution, script, code
 from IPython.testing import decorators as dec
 from IPython.testing import tools as tt
@@ -40,6 +39,8 @@ else:
     from StringIO import StringIO
 
 
+_ip = get_ipython()
+
 @magic.magics_class
 class DummyMagics(magic.Magics): pass
 
@@ -79,6 +80,35 @@ def test_extract_symbols_raises_exception_with_non_python_code():
     with nt.assert_raises(SyntaxError):
         code.extract_symbols(source, "hello")
 
+
+def test_magic_not_found():
+    # magic not found raises UsageError
+    with nt.assert_raises(UsageError):
+        _ip.magic('doesntexist')
+
+    # ensure result isn't success when a magic isn't found
+    result = _ip.run_cell('%doesntexist')
+    assert isinstance(result.error_in_exec, UsageError)
+
+
+def test_cell_magic_not_found():
+    # magic not found raises UsageError
+    with nt.assert_raises(UsageError):
+        _ip.run_cell_magic('doesntexist', 'line', 'cell')
+
+    # ensure result isn't success when a magic isn't found
+    result = _ip.run_cell('%%doesntexist')
+    assert isinstance(result.error_in_exec, UsageError)
+
+
+def test_magic_error_status():
+    def fail(shell):
+        1/0
+    _ip.register_magic_function(fail)
+    result = _ip.run_cell('%fail')
+    assert isinstance(result.error_in_exec, ZeroDivisionError)
+
+
 def test_config():
     """ test that config magic does not raise
     can happen if Configurable init is moved too early into
@@ -90,7 +120,6 @@ def test_config():
 
 def test_rehashx():
     # clear up everything
-    _ip = get_ipython()
     _ip.alias_manager.clear_aliases()
     del _ip.db['syscmdlist']
     
@@ -568,6 +597,11 @@ def test_timeit_quiet():
     with tt.AssertNotPrints("loops"):
         _ip.run_cell("%timeit -n1 -r1 -q 1")
 
+def test_timeit_return_quiet():
+    with tt.AssertNotPrints("loops"):
+        res = _ip.run_line_magic('timeit', '-n1 -r1 -q -o 1')
+    assert (res is not None)
+
 @dec.skipif(sys.version_info[0] >= 3, "no differences with __future__ in py3")
 def test_timeit_futures():
     "Test %timeit with __future__ environments"
@@ -578,6 +612,9 @@ def test_timeit_futures():
     ip.compile.reset_compiler_flags()
     with tt.AssertNotPrints('0.25'):
         ip.run_line_magic('timeit', '-n1 -r1 print(1/4)')
+def test_timeit_invalid_return():
+    with nt.assert_raises_regexp(SyntaxError, "outside function"):
+        _ip.run_line_magic('timeit', 'return')
 
 @dec.skipif(execution.profile is None)
 def test_prun_special_syntax():
@@ -601,13 +638,16 @@ def test_prun_quotes():
     nt.assert_equal(_ip.user_ns['x'], '\t')
 
 def test_extension():
-    tmpdir = TemporaryDirectory()
-    orig_ipython_dir = _ip.ipython_dir
+    # Debugging information for failures of this test
+    print('sys.path:')
+    for p in sys.path:
+        print(' ', p)
+    print('CWD', os.getcwd())
+
+    nt.assert_raises(ImportError, _ip.magic, "load_ext daft_extension")
+    daft_path = os.path.join(os.path.dirname(__file__), "daft_extension")
+    sys.path.insert(0, daft_path)
     try:
-        _ip.ipython_dir = tmpdir.name
-        nt.assert_raises(ImportError, _ip.magic, "load_ext daft_extension")
-        url = os.path.join(os.path.dirname(__file__), "daft_extension.py")
-        _ip.magic("install_ext %s" % url)
         _ip.user_ns.pop('arq', None)
         invalidate_caches()   # Clear import caches
         _ip.magic("load_ext daft_extension")
@@ -615,22 +655,18 @@ def test_extension():
         _ip.magic("unload_ext daft_extension")
         assert 'arq' not in _ip.user_ns
     finally:
-        _ip.ipython_dir = orig_ipython_dir
-        tmpdir.cleanup()
+        sys.path.remove(daft_path)
 
 
-# The nose skip decorator doesn't work on classes, so this uses unittest's skipIf
-@skipIf(dec.module_not_available('IPython.nbformat'), 'nbformat not importable')
-class NotebookExportMagicTests(TestCase):
-    def test_notebook_export_json(self):
-        _ip = get_ipython()
-        _ip.history_manager.reset()   # Clear any existing history.
-        cmds = [u"a=1", u"def b():\n  return a**2", u"print('noël, été', b())"]
-        for i, cmd in enumerate(cmds, start=1):
-            _ip.history_manager.store_inputs(i, cmd)
-        with TemporaryDirectory() as td:
-            outfile = os.path.join(td, "nb.ipynb")
-            _ip.magic("notebook -e %s" % outfile)
+def test_notebook_export_json():
+    _ip = get_ipython()
+    _ip.history_manager.reset()   # Clear any existing history.
+    cmds = [u"a=1", u"def b():\n  return a**2", u"print('noël, été', b())"]
+    for i, cmd in enumerate(cmds, start=1):
+        _ip.history_manager.store_inputs(i, cmd)
+    with TemporaryDirectory() as td:
+        outfile = os.path.join(td, "nb.ipynb")
+        _ip.magic("notebook -e %s" % outfile)
 
 
 class TestEnv(TestCase):
@@ -996,3 +1032,12 @@ def test_ls_magic():
         j = json_formatter(lsmagic)
     nt.assert_equal(sorted(j), ['cell', 'line'])
     nt.assert_equal(w, []) # no warnings
+
+def test_strip_initial_indent():
+    def sii(s):
+        lines = s.splitlines()
+        return '\n'.join(code.strip_initial_indent(lines))
+
+    nt.assert_equal(sii("  a = 1\nb = 2"), "a = 1\nb = 2")
+    nt.assert_equal(sii("  a\n    b\nc"), "a\n  b\nc")
+    nt.assert_equal(sii("a\n  b"), "a\n  b")
